/* To use this logger, just require the file and use the logger as follows:
 * <code>
 * const logger = require('./common/logger.js');
 * logger.log('info', 'This is an information message.');
 * logger.log('debug', 'This is an information message.');
 * logger.log('warn', 'This is an information message.');
 * logger.log('warning', 'This is an information message.');
 * logger.log('error', 'This is an information message.');
 * </code>
 */

const winston = require('winston');
const fs = require('fs');
const config = require('./config.js');

const __loggerCustomLevels = {
    label: 'default',
    levels: {
        debug: 3,
        info: 2,
        warning: 1,
        error: 0,
    },
    colors: {
        debug: 'blue',
        info: 'green',
        warn: 'yellow',
        warning: 'yellow',
        error: 'red',
    },
};

const __logFile = `${require('os').tmpdir()}/amadeus.log`;
fs.open(__logFile, 'w', function (err, file) {
    if (err) throw err;
});

var __logger = winston.createLogger({
    transports: [
        new winston.transports.File({
            eol: '\n',
            filename: __logFile,
            maxsize: '10M',
            tailable: true,
        }),
    ],
    level: `${config.loglevel}`,
    json: false,
    handleExceptions: true,
    levels: __loggerCustomLevels.levels,
    format: winston.format.combine(
        winston.format.timestamp({ format: 'YYYY-MM-DD HH:mm:ss' }),
        winston.format.label(),
        winston.format.colorize({ all: true }),
        winston.format.simple(),
        winston.format.printf(({ level, message, label, timestamp }) => {
            return `[${timestamp}] ${message}`;
        })
    ),
});

var __nopLektord = data => {};
var __lektord = data => {
    data.toString()
        .split(/\r?\n/)
        .forEach(line => {
            var line = line.trim();
            if (!line) return;
            switch (line[0]) {
                case '*':
                    logger.warn('lektord', line.replace(/^\* (.*)$/g, '$1'));
                    break;
                case '!':
                    logger.error('lektord', line.replace(/^! (.*)$/g, '$1'));
                    break;
                case '.':
                    if (line.length >= 2 && line[1] != '.') {
                        logger.info('lektord', line.replace(/^\. (.*)$/g, '$1'));
                        break;
                    }
                /* Fallthrough case */
                default:
                    logger.debug('lektord', line.replace(/^\.\.\. (.*)$/g, '\t$1'));
                    break;
            }
        });
};

var logger = {
    logfile: __logFile,
    lektord: config.loglektord ? __lektord : __nopLektord,
    debug: (cat, lg) => {
        __logger.debug(`${cat.padEnd(10)} | ${lg}`);
    },
    info: (cat, lg) => {
        __logger.info(`${cat.padEnd(10)} | ${lg}`);
    },
    warn: (cat, lg) => {
        __logger.warning(`${cat.padEnd(10)} | ${lg}`);
    },
    warning: (cat, lg) => {
        __logger.warning(`${cat.padEnd(10)} | ${lg}`);
    },
    error: (cat, lg) => {
        __logger.error(`${cat.padEnd(10)} | ${lg}`);
    },
};

module.exports = global.logger = module.exports = logger;
