/* The main window instance */

const { ipcRenderer } = require('electron'),
    logger = require('../common/logger.js'),
    ejs = require('ejs'),
    e = require('express');

/* prettier-ignore */
const buttonList = [
    [ 'left',   `<i class="fas fa-play"></i>`,          'commandPlay',      'Play or pause Lektor'  ],
    [ 'left',   `<i class="fas fa-stop"></i>`,          'commandStop',      'Stop lektor'           ],
    [ 'left',   `<i class="fas fa-eraser"></i>`,        'commandClear',     'Clear the queue'       ],

    [ 'sleft',  `<i class="fas fa-search"></i>`,        'selectAdvence',    'Advence search'        ],
    [ 'sleft',  `<i class="fas fa-database"></i>`,      'selectDatabase',   'Database search'       ],
    [ 'sleft',  `<i class="fas fa-tag"></i>`,           'selectPlaylist',   'Playlist search'       ],
    [ 'sleft',  `<i class="fas fa-bookmark"></i>`,      'selectPool',       'Pool search'           ],

    [ 'right',  `<i class="fas fa-compass"></i>`,       'openKurisu',       'Kurisu in a new page'  ],
    [ 'right',  `<i class="fas fa-user-friends"></i>`,  'openUserView',     'Open client view'      ],
    [ 'right',  `<i class="fas fa-music"></i>`,         'openMdtView',      'Open metadata view'    ],
    [ 'right',  `<i class="fas fa-sync-alt"></i>`,      'reloadQueue',      'Reload the queue'      ],
    [ 'right',  `<i class="fas fa-window-close"></i>`,  'closeButton',      'Quit Amadeus'          ],
];

/* Global variables */
var countKaraInQueue = 0;
var currentSong = 0;
var dragCounter = 0;
var leavedElement;
var isDnDFromDB = false;

function addIpcToButton(btnId, ipc) {
    document.getElementById(btnId).addEventListener('click', () => {
        ipc.forEach(ipc => {
            ipcRenderer.send(ipc);
        });
    });
}

function autoFillSettings() {
    $('#inputHost').val(config.host);
    $('#inputPort').val(config.port);
    $('#inputAttachMode').prop('checked', true);
    $('#inputDbPath').val(config.database);
    $('#inputLogLektord').prop('checked', config.loglektord);
    $(`#log${config.loglevel.toUpperCase()}`).prop('checked', true);
}

function flushFillSettings() {
    const set = {
        host: $('#inputHost').val(),
        port: $('#inputPort').val(),
        attachmode: $('#inputAttachMode').prop('checked'),
        db: $('#inputDbPath').val(),
        loglektord: $('#inputLogLektord').prop('checked'),
        loglevel: {
            ERROR: $('#logERROR').prop('checked'),
            WARNING: $('#logWARNING').prop('checked'),
            INFO: $('#logINFO').prop('checked'),
            DEBUG: $('#logDEBUG').prop('checked'),
        },
    };
    ipcRenderer.send('set-settings', set);
}

window.onload = () => {
    createButtonList(buttonList);

    addIpcToButton('closeButton', ['close-app']);

    addIpcToButton('commandPlay', ['cmd-play']);
    addIpcToButton('commandStop', ['cmd-stop']);
    addIpcToButton('commandClear', ['cmd-clear']);

    addIpcToButton('selectAdvence', ['select-advence']);
    addIpcToButton('selectDatabase', ['select-database', 'reload-db-request']);
    addIpcToButton('selectPlaylist', ['select-playlist']);
    addIpcToButton('selectPool', ['select-pool']);
    addIpcToButton('openMdtView', ['select-mdt']);

    addIpcToButton('openKurisu', ['toggle-kurisu']);
    addIpcToButton('openUserView', ['toggle-client-view']);
    addIpcToButton('reloadQueue', ['reload-queue-request']);

    setTimeout(() => ipcRenderer.send('reload-queue-request'), 1000);
    setInterval(() => ipcRenderer.send('verify-queue-reloaded-request'), 50);
    setInterval(() => ipcRenderer.send('get-song-time-data'), 50);
    logger.debug('instance', 'Window loaded');

    $('#filterInput').on('keypress', e => {
        /* On 'Return'. */
        if (e.which != 13) return;
        ipcRenderer.send('reload-db-request', {
            search: $('#filterInput').val(),
        });
    });

    document.addEventListener(
        'keydown',
        e => {
            if (e.ctrlKey && e.keyCode == 70) {
                logger.debug('instance', 'Focus find bar');
                $('#filterInput').focus();
            }
        },
        false
    );

    /* Autofill settings */
    autoFillSettings();
};

/* Create the button list */
function createButtonList(list) {
    function action(ipc, argument) {
        logger.error('instance', 'The action function builder for control buttons is not implemented');
    }
    var renderHtmlLeft = '';
    var renderHtmlRight = '';
    var renderHtmlSLeft = '';
    list.forEach(btn => {
        if (btn[0] == 'left') {
            renderHtmlLeft = `${renderHtmlLeft}
                              <button id="${btn[2]}" type="button" class="btn btn-primary p-2 bd-highlight"
                                      data-toggle="tooltip" data-placement="bottom" title="${btn[3]}"
                                      style="-webkit-app-region: no-drag">${btn[1]}</button>`;
            logger.debug('instance', `Create button for "${btn[1]}" with id "${btn[2]} at the left"`);
        } else if (btn[0] == 'right') {
            renderHtmlRight = `${renderHtmlRight}
                               <button id="${btn[2]}" type="button" class="btn btn-secondary p-2 bd-highlight"
                                       data-toggle="tooltip" data-placement="bottom" title="${btn[3]}"
                                       style="-webkit-app-region: no-drag">${btn[1]}</button>`;
            logger.debug('instance', `Create button for "${btn[1]}" with id "${btn[2]} at the right"`);
        } else if (btn[0] == 'sleft') {
            renderHtmlSLeft = `${renderHtmlSLeft}
                               <button id="${btn[2]}" type="button" class="btn btn-secondary p-2 bd-highlight"
                                       data-toggle="tooltip" data-placement="bottom" title="${btn[3]}"
                                       style="-webkit-app-region: no-drag">${btn[1]}</button>`;
            logger.debug('instance', `Create button for "${btn[1]}" with id "${btn[2]} at the right"`);
        } else {
            logger.warn('instance', `Unknown button type ${btn[0]} in top bar builder`);
        }
    });
    document.getElementById('buttonPanelListLeft').innerHTML = `
        <div class="btn-group" role="group">${renderHtmlLeft}</div>
        <div class="btn-group" role="group">${renderHtmlSLeft}</div>
        <div class="p-1 bd-highlight mb-1 mr-auto ml-auto" style="width: 60%;-webkit-app-region: no-drag;">
            <input id="filterInput" type="text" class="form-control filterInput" placeholder="Filter...">
        </div>
        <div class="btn-group" role="group">
        <button id="btn-settings" type="button" class="btn btn-secondary p-2 bd-highlight"
                style="-webkit-app-region: no-drag" data-toggle="tooltip"
                data-placement="bottom" title="Open settings">
            <i class="fas fa-cogs"></i>
        </button>
        ${renderHtmlRight}
        </div>`;

    /* Set things up for the settings modal*/
    var modal = document.getElementById('modal-settings');
    var btn = document.getElementById('btn-settings');
    var span = document.getElementsByClassName('modal-close')[0];

    btn.onclick = function () {
        modal.style.display = 'block';
    };

    span.onclick = function () {
        modal.style.display = 'none';
        flushFillSettings();
    };

    window.onclick = function (event) {
        if (event.target == modal) {
            modal.style.display = 'none';
        }
    };
}

/* Create the left panel */
ipcRenderer.on('reload-db-responce', (event, arg) => {
    logger.debug('instance', `Web page got reload-db`);
    document.getElementById('panelLeft').innerHTML = '';
    arg.forEach(kara => {
        ejs.renderFile(__dirname + '/views/karaDBListItem.ejs', { kara: kara }, (err, data) => {
            if (err) logger.error('instance', err);
            document.getElementById('panelLeft').innerHTML = `
                ${document.getElementById('panelLeft').innerHTML}
                ${data}`;
        });
    });
    [].forEach.call(document.querySelectorAll('#panelLeft .karaCard'), addDBKaraEventHandlers);
});

/* Create the right panel: the queue */
ipcRenderer.on('reload-queue-responce', (event, arg) => {
    logger.debug('instance', `Web page got reload-queue`);
    countKaraInQueue = 0;
    document.getElementById('panelRight').innerHTML = '';
    arg.forEach(kara => {
        countKaraInQueue++;
        ejs.renderFile(__dirname + '/views/karaQueueListItem.ejs', { kara: kara }, (err, data) => {
            if (err) logger.error('instance', err);
            document.getElementById('panelRight').innerHTML = `
                ${document.getElementById('panelRight').innerHTML}
                ${data}`;
        });
    });
    [].forEach.call(document.querySelectorAll('#panelRight .karaCard'), addQueueKaraEventHandlers);
});

ipcRenderer.on('send-song-time-data', (event, timeData) => {
    document.getElementById('progressBar').style.width = `${
        (timeData.elapsed / timeData.total) * document.documentElement.clientWidth
    }px`;
    if (timeData.state == 'play' || timeData.state == 'pause') {
        if (currentSong != timeData.song) {
            document.getElementsByClassName('karaQueue')[currentSong].style.background = '#4e5d6c';
            currentSong = timeData.song;
        }
        document.getElementsByClassName('karaQueue')[timeData.song].style.background = '#6b7d8e';
    } else {
        if (document.getElementsByClassName('karaQueue')[timeData.song])
            document.getElementsByClassName('karaQueue')[timeData.song].style.background = '#4e5d6c';
    }
});

function addDBKaraEventHandlers(element) {
    element.addEventListener('dragstart', onDragStartDB, false);
    element.addEventListener('dragend', onDragEnd, false);
    element.getElementsByClassName('karaAddBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('add-kara-queue-id', {
            id: element.getElementsByClassName('karaID')[0].innerText,
        })
    );
    element.getElementsByClassName('karaInsertBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('insert-kara-queue-id', {
            id: element.getElementsByClassName('karaID')[0].innerText,
        })
    );
}

function addQueueKaraEventHandlers(element) {
    element.addEventListener('dragstart', onDragStartQueue, false);
    element.addEventListener('dragenter', onDragEnter, false);
    element.addEventListener('dragover', onDragOver, false);
    element.addEventListener('dragleave', ondragLeave, false);
    element.addEventListener('drop', onDrop, false);
    element.addEventListener('dragend', onDragEnd, false);
    element.getElementsByClassName('karaDeleteBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('delete-kara-queue-pos', {
            position: element.getElementsByClassName('karaPosition')[0].innerText,
        })
    );
    element.getElementsByClassName('karaPlayBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('play-kara-queue-pos', {
            position: element.getElementsByClassName('karaPosition')[0].innerText,
        })
    );
}

function onDragStartDB(event) {
    isDnDFromDB = true;
    event.dataTransfer.effectAllowed = 'move';
    event.currentTarget.style.opacity = 0.4;
    event.dataTransfer.setData('text/plain', event.currentTarget.getElementsByClassName('karaID')[0].innerText);
}

function onDragStartQueue(event) {
    event.dataTransfer.effectAllowed = 'move';
    event.currentTarget.style.opacity = 0.4;
    event.dataTransfer.setData('text/plain', event.currentTarget.getElementsByClassName('karaPosition')[0].innerText);
}

function onDragEnter(event) {
    if (event.currentTarget) {
        if (leavedElement != event.currentTarget) {
            leavedElement.style.borderTop = '';
        }
        event.currentTarget.style.borderTop = '3px solid grey';
        dragCounter++;
    }
}

function onDragOver(event) {
    if (event.preventDefault) {
        event.preventDefault();
    }
    event.dataTransfer.dropEffect = 'move';
}

function ondragLeave(event) {
    dragCounter--;
    leavedElement = event.currentTarget;
    if (dragCounter == 0) {
        event.currentTarget.style.borderTop = '';
    }
}

function onDrop(event) {
    event.currentTarget.style.borderTop = '';
    if (isDnDFromDB) {
        ipcRenderer.send('add-kara-queue-pos', {
            id: event.dataTransfer.getData('text/plain'),
            position: event.currentTarget.getElementsByClassName('karaPosition')[0].innerText,
            queueSize: countKaraInQueue,
        });
    } else {
        ipcRenderer.send('queue-moved-kara', {
            from: event.dataTransfer.getData('text/plain'),
            to: event.currentTarget.getElementsByClassName('karaPosition')[0].innerText,
        });
    }
    isDnDFromDB = false;
    return false;
}

function onDragEnd(event) {
    event.currentTarget.style.opacity = 1;
    dragCounter = 0;
}
